/************************************************************
 *  Martin Toepfer     ***     PROJECT : Lingvisto          *
 *  Wissmannstr. 5     ***                                  *
 *  D - 22041 Hamburg  ***     File    : xawfileselect.c    *
 *  Germany            ***     Date    : 04.04.1997         *
 ************************************************************/
#include <dirent.h>
#include <limits.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/List.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Viewport.h>
#include "datastructure.h"
#include "error.h"
#include "file.h"
#include "xawfileselect.h"
#include "xawfunction.h"

/************************************************************
 * define PATH_MAX if necessary                             *
 ************************************************************/
#ifndef PATH_MAX
#define PATH_MAX 1024
#endif
   
/************************************************************
 *             global variables (in xaw.c)                  *
 ************************************************************/
extern Boolean b;
extern Widget top;

/************************************************************
 *               some more global variables                 *
 ************************************************************/
static char *flist[5000];
static char directory[PATH_MAX + 1];
static Widget input, list, pwd; 

/************************************************************
 * DisplayDirName : show name of current working directory  *
 ************************************************************/
static void DisplayDirName(void) {
  getcwd(directory, PATH_MAX + 1);
  XtVaSetValues(pwd, XtNlabel, directory, NULL);
}

/************************************************************
 * DestroyList : frees memory in listitems[]                *
 *                                                          *
 * flist : list to free                                     *
 ************************************************************/
static void DestroyList(char *flist[]) {
  int i;
  for(i = 0; flist[i] != NULL; free(flist[i++]))
    ;
}
      
/************************************************************
 * CreateList : reads directory in listitems[]              *
 *                                                          *
 * flist : list to put strings into                         *
 ************************************************************/
static void CreateList(char *flist[]) {
  DIR *dp;
  struct dirent *dirp;
  struct stat buf;
  int i = 2;

  /*
   * The first two entries are the directories . and ..
   * so we are always able to enter the upper directory.
   * This is necessary for directories with x-permission
   * but no r-permission, because chdir(2) returns no error
   * if we enter such a directory
   */

  flist[0] = malloc(strlen(".") + 1);
  flist[1] = malloc(strlen("..") + 1);
  if ((flist[0] != NULL) && (flist[1] != NULL)) {
    strcpy(flist[0], ".");
    strcpy(flist[1], "..");
  }
  else {
    lvo_error(NO_MEMORY);
    return;
  }

  /*
   * lets read directory entries
   */

  dp = opendir(".");
  while((dirp = readdir(dp)) != NULL) {
    if (lstat(dirp->d_name, &buf) >= 0) {
      int len = strlen(dirp->d_name) + 2;

      /*
       * filter out hidden files
       */
      
      if (dirp->d_name[0] != '.') {
	
	/*
	 * copy filename into tmp and insert into flist
	 */
	
	char *tmp = (char *) malloc((size_t) len);
	if (tmp != NULL) {
	  strcpy(tmp, dirp->d_name);
	  
	  /*
	   * file is directory -> append '/'
	   */
	  
	  if (S_ISDIR(buf.st_mode))
	    tmp = strcat(tmp, "/");
	  flist[i++] = tmp;
	}
	else
	  lvo_error(NO_MEMORY);
      }
    }
  }
  closedir(dp);
  flist[i] = NULL;
}

/************************************************************
 * SelectFile : displays fileselectorbox, returns filename  *
 ************************************************************/
/*************************************************
 *         'Cancel' button callback              *
 *************************************************/
/* ARGSUSED */
static void Cancel(Widget w, XtPointer x, XtPointer y) {
  b = (Boolean) False;
  DestroyList(flist);
  XtDestroyWidget(XtParent(XtParent(w)));
  SetAllSensitive((Boolean) True);
  XtSetKeyboardFocus(top, top);
}

/*************************************************
 *       'Load'/'Save' button callback           *
 *************************************************/
/* ARGSUSED */
void Load(Widget w, XtPointer x, XtPointer y) {
  String f;
  XtVaGetValues(input, XtNstring, &f, NULL);
  set_filename(f);
  Cancel(w, NULL, NULL);
}
   
/*************************************************
 *            listWidget callback                *
 *************************************************/
/* ARGSUSED */
static void Select(Widget w, XtPointer x, XtPointer y) {
  struct stat st;
  XawListReturnStruct *s = (XawListReturnStruct *) y;
  
  /*
   * get file information
   */
  
  if(lstat(s->string, &st) != -1) {
    if(S_ISDIR(st.st_mode)) {
      
      /*
       * file is directory
       */
      
      if(chdir(s->string) != -1) {
	DisplayDirName();
	DestroyList(flist);
	CreateList(flist);
	XawListChange(list, flist, -1, -1, True);
      }
    }
    else {
      
      /*
       * file is not a directory
       */
      
      set_filename(s->string);
      Cancel(XtParent(w), NULL, NULL);
    }
  }
}

/*************************************************
 *            begin of SelectFile()              *
 *************************************************/
void SelectFile(int mode) {
  Widget psh, form, dir, vp, load, cancel, mess;
  String transL = "<Key>Return: LoadAction()";
  
  /*
   * this is the popup shell
   */
  
  psh = ComputePsh();
  
  /*
   * this is the fileselector formWidget
   */
  
  form = XtVaCreateManagedWidget("", formWidgetClass, psh, NULL);
  
  /*
   * this is the labelWidget that displays "Current Directory :"
   */
  
  dir = XtVaCreateManagedWidget("", labelWidgetClass, form, NULL);
  XtVaSetValues(dir, 
		XtNlabel, "Current Directory :", 
		XtNborderWidth, 0,
		NULL);
  
  /*
   * this is the labelWidget that displays the current directory
   */
  
  pwd = XtVaCreateManagedWidget("", labelWidgetClass, form, NULL);
  DisplayDirName();
  XtVaSetValues(pwd,
		XtNwidth, 400,
		XtNborderWidth, 0,
		XtNfromVert, dir,
		XtNjustify, XtJustifyLeft,
		NULL);
  
  /* 
   * this is the viewportWidget that contains the file list
   */
  
  vp = XtVaCreateManagedWidget("", viewportWidgetClass, form, NULL);
  XtVaSetValues(vp, 
		XtNheight, 250, 
		XtNwidth, 250,
		XtNfromVert, pwd,
		XtNvertDistance, 10,
		XtNuseBottom, True,
		XtNuseRight, True,
		XtNforceBars, False,
		XtNallowHoriz, True,
		XtNallowVert, True,
		NULL);
  
  /*
   * this is the 'Load'/'Save' commandWidget
   */
  
  load = XtVaCreateManagedWidget("", commandWidgetClass, form, NULL);
  XtAddCallback(load, XtNcallback, Load, NULL);
  XtVaSetValues(load,
		XtNfromVert, pwd,
		XtNvertDistance, 10,
		XtNfromHoriz, vp,
		XtNhorizDistance, 20,
		NULL);
  if (mode == LOAD)
    XtVaSetValues(load, XtNlabel, " Load ", NULL);
  else
    XtVaSetValues(load, XtNlabel, " Save ", NULL);
  
  /*
   * this is the 'Cancel' commandWidget
   */
  
  cancel = XtVaCreateManagedWidget("Cancel", commandWidgetClass, form, NULL);
  XtAddCallback(cancel, XtNcallback, Cancel, NULL);
  XtVaSetValues(cancel,
		XtNfromVert, load,
		XtNfromHoriz, vp,
		XtNhorizDistance, 20,
		NULL);
  
  /*
   * this is the listWidget that displays the file list
   */
  
  list = XtVaCreateManagedWidget("", listWidgetClass, vp, NULL);
  CreateList(flist);
  XawListChange(list, flist, -1, -1, True);
  XtAddCallback(list, XtNcallback, Select, NULL);
  XtVaSetValues(list,
		XtNdefaultColumns, 1,
		XtNforceColumns, True,
		XtNrowSpacing, 4,
		NULL);
  
  /*
   * this is the labelWidget that displays "Please insert filename :"
   */
  
  mess = XtVaCreateManagedWidget("", labelWidgetClass, form, NULL);
  XtVaSetValues(mess,
		XtNfromVert, vp,
		XtNvertDistance, 10,
		XtNlabel, "Please insert filename :",
		XtNborderWidth, 0,
		NULL);
  
  /*
   * this is the asciiTextWidget where you may insert a filename
   */
  
  input = XtVaCreateManagedWidget("", asciiTextWidgetClass, form, NULL);
  XtVaSetValues(input,
		XtNfromVert, mess,
		XtNwidth, 400,
		XtNeditType, XawtextEdit,
		XtNdisplayCaret, True,
		NULL);
  
  /*
   * now lets popup the shell
   */
  
  XtManageChild(form);
  XtPopup(psh, XtGrabNonexclusive);
  XtSetKeyboardFocus(psh, input);
  XtSetKeyboardFocus(top, input);
  XtOverrideTranslations(input, XtParseTranslationTable(transL));
  
  /*
   * wait until button is pressed
   */
  
  b = (Boolean) True;
  while(b == (Boolean) True)
    DoNextEvent();
}
